@@ -1,4 +1,4 @@
-#line 1 "/source/test-c/test-alignment.c"
+#line 1 "/home/sigscheme/source/test-c/test-alignment.c"
 /*===========================================================================
  *  Filename : test-alignment.c
  *  About    : unit test for alignment on stack
